# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class SUPPORTEDBASEIMAGESENUM(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    allowed enum values
    """
    RAY_ML_1_2_0 = "anyscale/ray-ml:1.2.0"
    RAY_ML_1_2_0_GPU = "anyscale/ray-ml:1.2.0-gpu"
    RAY_ML_1_3_0 = "anyscale/ray-ml:1.3.0"
    RAY_ML_1_3_0_GPU = "anyscale/ray-ml:1.3.0-gpu"
    RAY_ML_1_3_0_PY36 = "anyscale/ray-ml:1.3.0-py36"
    RAY_ML_1_3_0_PY36_GPU = "anyscale/ray-ml:1.3.0-py36-gpu"
    RAY_ML_1_3_0_PY37 = "anyscale/ray-ml:1.3.0-py37"
    RAY_ML_1_3_0_PY37_GPU = "anyscale/ray-ml:1.3.0-py37-gpu"
    RAY_ML_1_3_0_PY38 = "anyscale/ray-ml:1.3.0-py38"
    RAY_ML_1_3_0_PY38_GPU = "anyscale/ray-ml:1.3.0-py38-gpu"
    RAY_ML_1_4_RC0 = "anyscale/ray-ml:1.4-rc0"
    RAY_ML_1_4_RC0_GPU = "anyscale/ray-ml:1.4-rc0-gpu"
    RAY_ML_1_4_RC0_PY36 = "anyscale/ray-ml:1.4-rc0-py36"
    RAY_ML_1_4_RC0_PY36_GPU = "anyscale/ray-ml:1.4-rc0-py36-gpu"
    RAY_ML_1_4_RC0_PY37 = "anyscale/ray-ml:1.4-rc0-py37"
    RAY_ML_1_4_RC0_PY37_GPU = "anyscale/ray-ml:1.4-rc0-py37-gpu"
    RAY_ML_1_4_RC0_PY38 = "anyscale/ray-ml:1.4-rc0-py38"
    RAY_ML_1_4_RC0_PY38_GPU = "anyscale/ray-ml:1.4-rc0-py38-gpu"
    RAY_ML_1_4_RC1 = "anyscale/ray-ml:1.4-rc1"
    RAY_ML_1_4_RC1_GPU = "anyscale/ray-ml:1.4-rc1-gpu"
    RAY_ML_1_4_RC1_PY36 = "anyscale/ray-ml:1.4-rc1-py36"
    RAY_ML_1_4_RC1_PY36_GPU = "anyscale/ray-ml:1.4-rc1-py36-gpu"
    RAY_ML_1_4_RC1_PY37 = "anyscale/ray-ml:1.4-rc1-py37"
    RAY_ML_1_4_RC1_PY37_GPU = "anyscale/ray-ml:1.4-rc1-py37-gpu"
    RAY_ML_1_4_RC1_PY38 = "anyscale/ray-ml:1.4-rc1-py38"
    RAY_ML_1_4_RC1_PY38_GPU = "anyscale/ray-ml:1.4-rc1-py38-gpu"
    RAY_ML_1_4_RC2 = "anyscale/ray-ml:1.4-rc2"
    RAY_ML_1_4_RC2_GPU = "anyscale/ray-ml:1.4-rc2-gpu"
    RAY_ML_1_4_RC2_PY36 = "anyscale/ray-ml:1.4-rc2-py36"
    RAY_ML_1_4_RC2_PY36_GPU = "anyscale/ray-ml:1.4-rc2-py36-gpu"
    RAY_ML_1_4_RC2_PY37 = "anyscale/ray-ml:1.4-rc2-py37"
    RAY_ML_1_4_RC2_PY37_GPU = "anyscale/ray-ml:1.4-rc2-py37-gpu"
    RAY_ML_1_4_RC2_PY38 = "anyscale/ray-ml:1.4-rc2-py38"
    RAY_ML_1_4_RC2_PY38_GPU = "anyscale/ray-ml:1.4-rc2-py38-gpu"
    RAY_ML_1_4_0 = "anyscale/ray-ml:1.4.0"
    RAY_ML_1_4_0_GPU = "anyscale/ray-ml:1.4.0-gpu"
    RAY_ML_1_4_0_PY36 = "anyscale/ray-ml:1.4.0-py36"
    RAY_ML_1_4_0_PY36_GPU = "anyscale/ray-ml:1.4.0-py36-gpu"
    RAY_ML_1_4_0_PY37 = "anyscale/ray-ml:1.4.0-py37"
    RAY_ML_1_4_0_PY37_GPU = "anyscale/ray-ml:1.4.0-py37-gpu"
    RAY_ML_1_4_0_PY38 = "anyscale/ray-ml:1.4.0-py38"
    RAY_ML_1_4_0_PY38_GPU = "anyscale/ray-ml:1.4.0-py38-gpu"
    RAY_ML_1_4_1 = "anyscale/ray-ml:1.4.1"
    RAY_ML_1_4_1_GPU = "anyscale/ray-ml:1.4.1-gpu"
    RAY_ML_1_4_1_PY36 = "anyscale/ray-ml:1.4.1-py36"
    RAY_ML_1_4_1_PY36_GPU = "anyscale/ray-ml:1.4.1-py36-gpu"
    RAY_ML_1_4_1_PY37 = "anyscale/ray-ml:1.4.1-py37"
    RAY_ML_1_4_1_PY37_GPU = "anyscale/ray-ml:1.4.1-py37-gpu"
    RAY_ML_1_4_1_PY38 = "anyscale/ray-ml:1.4.1-py38"
    RAY_ML_1_4_1_PY38_GPU = "anyscale/ray-ml:1.4.1-py38-gpu"
    RAY_ML_1_5_0 = "anyscale/ray-ml:1.5.0"
    RAY_ML_1_5_0_GPU = "anyscale/ray-ml:1.5.0-gpu"
    RAY_ML_1_5_0_PY36 = "anyscale/ray-ml:1.5.0-py36"
    RAY_ML_1_5_0_PY36_GPU = "anyscale/ray-ml:1.5.0-py36-gpu"
    RAY_ML_1_5_0_PY37 = "anyscale/ray-ml:1.5.0-py37"
    RAY_ML_1_5_0_PY37_GPU = "anyscale/ray-ml:1.5.0-py37-gpu"
    RAY_ML_1_5_0_PY38 = "anyscale/ray-ml:1.5.0-py38"
    RAY_ML_1_5_0_PY38_GPU = "anyscale/ray-ml:1.5.0-py38-gpu"
    RAY_ML_1_5_2 = "anyscale/ray-ml:1.5.2"
    RAY_ML_1_5_2_GPU = "anyscale/ray-ml:1.5.2-gpu"
    RAY_ML_1_5_2_PY36 = "anyscale/ray-ml:1.5.2-py36"
    RAY_ML_1_5_2_PY36_GPU = "anyscale/ray-ml:1.5.2-py36-gpu"
    RAY_ML_1_5_2_PY37 = "anyscale/ray-ml:1.5.2-py37"
    RAY_ML_1_5_2_PY37_GPU = "anyscale/ray-ml:1.5.2-py37-gpu"
    RAY_ML_1_5_2_PY38 = "anyscale/ray-ml:1.5.2-py38"
    RAY_ML_1_5_2_PY38_GPU = "anyscale/ray-ml:1.5.2-py38-gpu"
    RAY_ML_1_6_0 = "anyscale/ray-ml:1.6.0"
    RAY_ML_1_6_0_GPU = "anyscale/ray-ml:1.6.0-gpu"
    RAY_ML_1_6_0_PY36 = "anyscale/ray-ml:1.6.0-py36"
    RAY_ML_1_6_0_PY36_GPU = "anyscale/ray-ml:1.6.0-py36-gpu"
    RAY_ML_1_6_0_PY37 = "anyscale/ray-ml:1.6.0-py37"
    RAY_ML_1_6_0_PY37_GPU = "anyscale/ray-ml:1.6.0-py37-gpu"
    RAY_ML_1_6_0_PY38 = "anyscale/ray-ml:1.6.0-py38"
    RAY_ML_1_6_0_PY38_GPU = "anyscale/ray-ml:1.6.0-py38-gpu"
    RAY_ML_1_7_0 = "anyscale/ray-ml:1.7.0"
    RAY_ML_1_7_0_GPU = "anyscale/ray-ml:1.7.0-gpu"
    RAY_ML_1_7_0_PY36 = "anyscale/ray-ml:1.7.0-py36"
    RAY_ML_1_7_0_PY36_GPU = "anyscale/ray-ml:1.7.0-py36-gpu"
    RAY_ML_1_7_0_PY37 = "anyscale/ray-ml:1.7.0-py37"
    RAY_ML_1_7_0_PY37_GPU = "anyscale/ray-ml:1.7.0-py37-gpu"
    RAY_ML_1_7_0_PY38 = "anyscale/ray-ml:1.7.0-py38"
    RAY_ML_1_7_0_PY38_GPU = "anyscale/ray-ml:1.7.0-py38-gpu"
    RAY_ML_1_7_0RC0 = "anyscale/ray-ml:1.7.0rc0"
    RAY_ML_1_7_0RC0_GPU = "anyscale/ray-ml:1.7.0rc0-gpu"
    RAY_ML_1_7_0RC0_PY36 = "anyscale/ray-ml:1.7.0rc0-py36"
    RAY_ML_1_7_0RC0_PY36_GPU = "anyscale/ray-ml:1.7.0rc0-py36-gpu"
    RAY_ML_1_7_0RC0_PY37 = "anyscale/ray-ml:1.7.0rc0-py37"
    RAY_ML_1_7_0RC0_PY37_GPU = "anyscale/ray-ml:1.7.0rc0-py37-gpu"
    RAY_ML_1_7_0RC0_PY38 = "anyscale/ray-ml:1.7.0rc0-py38"
    RAY_ML_1_7_0RC0_PY38_GPU = "anyscale/ray-ml:1.7.0rc0-py38-gpu"
    RAY_ML_1_8_0 = "anyscale/ray-ml:1.8.0"
    RAY_ML_1_8_0_GPU = "anyscale/ray-ml:1.8.0-gpu"
    RAY_ML_1_8_0_PY36 = "anyscale/ray-ml:1.8.0-py36"
    RAY_ML_1_8_0_PY36_GPU = "anyscale/ray-ml:1.8.0-py36-gpu"
    RAY_ML_1_8_0_PY37 = "anyscale/ray-ml:1.8.0-py37"
    RAY_ML_1_8_0_PY37_GPU = "anyscale/ray-ml:1.8.0-py37-gpu"
    RAY_ML_1_8_0_PY38 = "anyscale/ray-ml:1.8.0-py38"
    RAY_ML_1_8_0_PY38_GPU = "anyscale/ray-ml:1.8.0-py38-gpu"
    RAY_ML_1_9_0_PY36_GPU = "anyscale/ray-ml:1.9.0-py36-gpu"
    RAY_ML_1_9_0_PY37_GPU = "anyscale/ray-ml:1.9.0-py37-gpu"
    RAY_ML_1_9_0_PY38_GPU = "anyscale/ray-ml:1.9.0-py38-gpu"
    RAY_ML_1_9_0RC1_PY36_GPU = "anyscale/ray-ml:1.9.0rc1-py36-gpu"
    RAY_ML_1_9_0RC1_PY37_GPU = "anyscale/ray-ml:1.9.0rc1-py37-gpu"
    RAY_ML_1_9_0RC1_PY38_GPU = "anyscale/ray-ml:1.9.0rc1-py38-gpu"
    RAY_ML_1_9_0RC2_PY36_GPU = "anyscale/ray-ml:1.9.0rc2-py36-gpu"
    RAY_ML_1_9_0RC2_PY37_GPU = "anyscale/ray-ml:1.9.0rc2-py37-gpu"
    RAY_ML_1_9_0RC2_PY38_GPU = "anyscale/ray-ml:1.9.0rc2-py38-gpu"
    RAY_ML_1_9_1RC0_PY36_GPU = "anyscale/ray-ml:1.9.1rc0-py36-gpu"
    RAY_ML_1_9_1RC0_PY37_GPU = "anyscale/ray-ml:1.9.1rc0-py37-gpu"
    RAY_ML_1_9_1RC0_PY38_GPU = "anyscale/ray-ml:1.9.1rc0-py38-gpu"
    RAY_ML_NIGHTLY_PY36_GPU = "anyscale/ray-ml:nightly-py36-gpu"
    RAY_ML_NIGHTLY_PY37_GPU = "anyscale/ray-ml:nightly-py37-gpu"
    RAY_ML_NIGHTLY_PY38_GPU = "anyscale/ray-ml:nightly-py38-gpu"
    RAY_1_2_0 = "anyscale/ray:1.2.0"
    RAY_1_2_0_GPU = "anyscale/ray:1.2.0-gpu"
    RAY_1_3_0 = "anyscale/ray:1.3.0"
    RAY_1_3_0_GPU = "anyscale/ray:1.3.0-gpu"
    RAY_1_3_0_PY36 = "anyscale/ray:1.3.0-py36"
    RAY_1_3_0_PY36_GPU = "anyscale/ray:1.3.0-py36-gpu"
    RAY_1_3_0_PY37 = "anyscale/ray:1.3.0-py37"
    RAY_1_3_0_PY37_GPU = "anyscale/ray:1.3.0-py37-gpu"
    RAY_1_3_0_PY38 = "anyscale/ray:1.3.0-py38"
    RAY_1_3_0_PY38_GPU = "anyscale/ray:1.3.0-py38-gpu"
    RAY_1_4_RC0 = "anyscale/ray:1.4-rc0"
    RAY_1_4_RC0_GPU = "anyscale/ray:1.4-rc0-gpu"
    RAY_1_4_RC0_PY36 = "anyscale/ray:1.4-rc0-py36"
    RAY_1_4_RC0_PY36_GPU = "anyscale/ray:1.4-rc0-py36-gpu"
    RAY_1_4_RC0_PY37 = "anyscale/ray:1.4-rc0-py37"
    RAY_1_4_RC0_PY37_GPU = "anyscale/ray:1.4-rc0-py37-gpu"
    RAY_1_4_RC0_PY38 = "anyscale/ray:1.4-rc0-py38"
    RAY_1_4_RC0_PY38_GPU = "anyscale/ray:1.4-rc0-py38-gpu"
    RAY_1_4_RC1 = "anyscale/ray:1.4-rc1"
    RAY_1_4_RC1_GPU = "anyscale/ray:1.4-rc1-gpu"
    RAY_1_4_RC1_PY36 = "anyscale/ray:1.4-rc1-py36"
    RAY_1_4_RC1_PY36_GPU = "anyscale/ray:1.4-rc1-py36-gpu"
    RAY_1_4_RC1_PY37 = "anyscale/ray:1.4-rc1-py37"
    RAY_1_4_RC1_PY37_GPU = "anyscale/ray:1.4-rc1-py37-gpu"
    RAY_1_4_RC1_PY38 = "anyscale/ray:1.4-rc1-py38"
    RAY_1_4_RC1_PY38_GPU = "anyscale/ray:1.4-rc1-py38-gpu"
    RAY_1_4_RC2 = "anyscale/ray:1.4-rc2"
    RAY_1_4_RC2_GPU = "anyscale/ray:1.4-rc2-gpu"
    RAY_1_4_RC2_PY36 = "anyscale/ray:1.4-rc2-py36"
    RAY_1_4_RC2_PY36_GPU = "anyscale/ray:1.4-rc2-py36-gpu"
    RAY_1_4_RC2_PY37 = "anyscale/ray:1.4-rc2-py37"
    RAY_1_4_RC2_PY37_GPU = "anyscale/ray:1.4-rc2-py37-gpu"
    RAY_1_4_RC2_PY38 = "anyscale/ray:1.4-rc2-py38"
    RAY_1_4_RC2_PY38_GPU = "anyscale/ray:1.4-rc2-py38-gpu"
    RAY_1_4_0 = "anyscale/ray:1.4.0"
    RAY_1_4_0_GPU = "anyscale/ray:1.4.0-gpu"
    RAY_1_4_0_PY36 = "anyscale/ray:1.4.0-py36"
    RAY_1_4_0_PY36_GPU = "anyscale/ray:1.4.0-py36-gpu"
    RAY_1_4_0_PY37 = "anyscale/ray:1.4.0-py37"
    RAY_1_4_0_PY37_GPU = "anyscale/ray:1.4.0-py37-gpu"
    RAY_1_4_0_PY38 = "anyscale/ray:1.4.0-py38"
    RAY_1_4_0_PY38_GPU = "anyscale/ray:1.4.0-py38-gpu"
    RAY_1_4_1 = "anyscale/ray:1.4.1"
    RAY_1_4_1_GPU = "anyscale/ray:1.4.1-gpu"
    RAY_1_4_1_PY36 = "anyscale/ray:1.4.1-py36"
    RAY_1_4_1_PY36_GPU = "anyscale/ray:1.4.1-py36-gpu"
    RAY_1_4_1_PY37 = "anyscale/ray:1.4.1-py37"
    RAY_1_4_1_PY37_GPU = "anyscale/ray:1.4.1-py37-gpu"
    RAY_1_4_1_PY38 = "anyscale/ray:1.4.1-py38"
    RAY_1_4_1_PY38_GPU = "anyscale/ray:1.4.1-py38-gpu"
    RAY_1_5_0 = "anyscale/ray:1.5.0"
    RAY_1_5_0_GPU = "anyscale/ray:1.5.0-gpu"
    RAY_1_5_0_PY36 = "anyscale/ray:1.5.0-py36"
    RAY_1_5_0_PY36_GPU = "anyscale/ray:1.5.0-py36-gpu"
    RAY_1_5_0_PY37 = "anyscale/ray:1.5.0-py37"
    RAY_1_5_0_PY37_GPU = "anyscale/ray:1.5.0-py37-gpu"
    RAY_1_5_0_PY38 = "anyscale/ray:1.5.0-py38"
    RAY_1_5_0_PY38_GPU = "anyscale/ray:1.5.0-py38-gpu"
    RAY_1_5_2 = "anyscale/ray:1.5.2"
    RAY_1_5_2_GPU = "anyscale/ray:1.5.2-gpu"
    RAY_1_5_2_PY36 = "anyscale/ray:1.5.2-py36"
    RAY_1_5_2_PY36_GPU = "anyscale/ray:1.5.2-py36-gpu"
    RAY_1_5_2_PY37 = "anyscale/ray:1.5.2-py37"
    RAY_1_5_2_PY37_GPU = "anyscale/ray:1.5.2-py37-gpu"
    RAY_1_5_2_PY38 = "anyscale/ray:1.5.2-py38"
    RAY_1_5_2_PY38_GPU = "anyscale/ray:1.5.2-py38-gpu"
    RAY_1_6_0 = "anyscale/ray:1.6.0"
    RAY_1_6_0_GPU = "anyscale/ray:1.6.0-gpu"
    RAY_1_6_0_PY36 = "anyscale/ray:1.6.0-py36"
    RAY_1_6_0_PY36_GPU = "anyscale/ray:1.6.0-py36-gpu"
    RAY_1_6_0_PY37 = "anyscale/ray:1.6.0-py37"
    RAY_1_6_0_PY37_GPU = "anyscale/ray:1.6.0-py37-gpu"
    RAY_1_6_0_PY38 = "anyscale/ray:1.6.0-py38"
    RAY_1_6_0_PY38_GPU = "anyscale/ray:1.6.0-py38-gpu"
    RAY_1_7_0 = "anyscale/ray:1.7.0"
    RAY_1_7_0_GPU = "anyscale/ray:1.7.0-gpu"
    RAY_1_7_0_PY36 = "anyscale/ray:1.7.0-py36"
    RAY_1_7_0_PY36_GPU = "anyscale/ray:1.7.0-py36-gpu"
    RAY_1_7_0_PY37 = "anyscale/ray:1.7.0-py37"
    RAY_1_7_0_PY37_GPU = "anyscale/ray:1.7.0-py37-gpu"
    RAY_1_7_0_PY38 = "anyscale/ray:1.7.0-py38"
    RAY_1_7_0_PY38_GPU = "anyscale/ray:1.7.0-py38-gpu"
    RAY_1_7_0RC0 = "anyscale/ray:1.7.0rc0"
    RAY_1_7_0RC0_GPU = "anyscale/ray:1.7.0rc0-gpu"
    RAY_1_7_0RC0_PY36 = "anyscale/ray:1.7.0rc0-py36"
    RAY_1_7_0RC0_PY36_GPU = "anyscale/ray:1.7.0rc0-py36-gpu"
    RAY_1_7_0RC0_PY37 = "anyscale/ray:1.7.0rc0-py37"
    RAY_1_7_0RC0_PY37_GPU = "anyscale/ray:1.7.0rc0-py37-gpu"
    RAY_1_7_0RC0_PY38 = "anyscale/ray:1.7.0rc0-py38"
    RAY_1_7_0RC0_PY38_GPU = "anyscale/ray:1.7.0rc0-py38-gpu"
    RAY_1_8_0 = "anyscale/ray:1.8.0"
    RAY_1_8_0_GPU = "anyscale/ray:1.8.0-gpu"
    RAY_1_8_0_PY36 = "anyscale/ray:1.8.0-py36"
    RAY_1_8_0_PY36_GPU = "anyscale/ray:1.8.0-py36-gpu"
    RAY_1_8_0_PY37 = "anyscale/ray:1.8.0-py37"
    RAY_1_8_0_PY37_GPU = "anyscale/ray:1.8.0-py37-gpu"
    RAY_1_8_0_PY38 = "anyscale/ray:1.8.0-py38"
    RAY_1_8_0_PY38_GPU = "anyscale/ray:1.8.0-py38-gpu"
    RAY_1_9_0_PY36 = "anyscale/ray:1.9.0-py36"
    RAY_1_9_0_PY36_CU101 = "anyscale/ray:1.9.0-py36-cu101"
    RAY_1_9_0_PY36_CU102 = "anyscale/ray:1.9.0-py36-cu102"
    RAY_1_9_0_PY36_CU110 = "anyscale/ray:1.9.0-py36-cu110"
    RAY_1_9_0_PY36_CU111 = "anyscale/ray:1.9.0-py36-cu111"
    RAY_1_9_0_PY36_CU112 = "anyscale/ray:1.9.0-py36-cu112"
    RAY_1_9_0_PY37 = "anyscale/ray:1.9.0-py37"
    RAY_1_9_0_PY37_CU101 = "anyscale/ray:1.9.0-py37-cu101"
    RAY_1_9_0_PY37_CU102 = "anyscale/ray:1.9.0-py37-cu102"
    RAY_1_9_0_PY37_CU110 = "anyscale/ray:1.9.0-py37-cu110"
    RAY_1_9_0_PY37_CU111 = "anyscale/ray:1.9.0-py37-cu111"
    RAY_1_9_0_PY37_CU112 = "anyscale/ray:1.9.0-py37-cu112"
    RAY_1_9_0_PY38 = "anyscale/ray:1.9.0-py38"
    RAY_1_9_0_PY38_CU101 = "anyscale/ray:1.9.0-py38-cu101"
    RAY_1_9_0_PY38_CU102 = "anyscale/ray:1.9.0-py38-cu102"
    RAY_1_9_0_PY38_CU110 = "anyscale/ray:1.9.0-py38-cu110"
    RAY_1_9_0_PY38_CU111 = "anyscale/ray:1.9.0-py38-cu111"
    RAY_1_9_0_PY38_CU112 = "anyscale/ray:1.9.0-py38-cu112"
    RAY_1_9_0RC1_PY36 = "anyscale/ray:1.9.0rc1-py36"
    RAY_1_9_0RC1_PY36_CU101 = "anyscale/ray:1.9.0rc1-py36-cu101"
    RAY_1_9_0RC1_PY36_CU102 = "anyscale/ray:1.9.0rc1-py36-cu102"
    RAY_1_9_0RC1_PY36_CU110 = "anyscale/ray:1.9.0rc1-py36-cu110"
    RAY_1_9_0RC1_PY36_CU111 = "anyscale/ray:1.9.0rc1-py36-cu111"
    RAY_1_9_0RC1_PY36_CU112 = "anyscale/ray:1.9.0rc1-py36-cu112"
    RAY_1_9_0RC1_PY37 = "anyscale/ray:1.9.0rc1-py37"
    RAY_1_9_0RC1_PY37_CU101 = "anyscale/ray:1.9.0rc1-py37-cu101"
    RAY_1_9_0RC1_PY37_CU102 = "anyscale/ray:1.9.0rc1-py37-cu102"
    RAY_1_9_0RC1_PY37_CU110 = "anyscale/ray:1.9.0rc1-py37-cu110"
    RAY_1_9_0RC1_PY37_CU111 = "anyscale/ray:1.9.0rc1-py37-cu111"
    RAY_1_9_0RC1_PY37_CU112 = "anyscale/ray:1.9.0rc1-py37-cu112"
    RAY_1_9_0RC1_PY38 = "anyscale/ray:1.9.0rc1-py38"
    RAY_1_9_0RC1_PY38_CU101 = "anyscale/ray:1.9.0rc1-py38-cu101"
    RAY_1_9_0RC1_PY38_CU102 = "anyscale/ray:1.9.0rc1-py38-cu102"
    RAY_1_9_0RC1_PY38_CU110 = "anyscale/ray:1.9.0rc1-py38-cu110"
    RAY_1_9_0RC1_PY38_CU111 = "anyscale/ray:1.9.0rc1-py38-cu111"
    RAY_1_9_0RC1_PY38_CU112 = "anyscale/ray:1.9.0rc1-py38-cu112"
    RAY_1_9_0RC2_PY36 = "anyscale/ray:1.9.0rc2-py36"
    RAY_1_9_0RC2_PY36_CU101 = "anyscale/ray:1.9.0rc2-py36-cu101"
    RAY_1_9_0RC2_PY36_CU102 = "anyscale/ray:1.9.0rc2-py36-cu102"
    RAY_1_9_0RC2_PY36_CU110 = "anyscale/ray:1.9.0rc2-py36-cu110"
    RAY_1_9_0RC2_PY36_CU111 = "anyscale/ray:1.9.0rc2-py36-cu111"
    RAY_1_9_0RC2_PY36_CU112 = "anyscale/ray:1.9.0rc2-py36-cu112"
    RAY_1_9_0RC2_PY37 = "anyscale/ray:1.9.0rc2-py37"
    RAY_1_9_0RC2_PY37_CU101 = "anyscale/ray:1.9.0rc2-py37-cu101"
    RAY_1_9_0RC2_PY37_CU102 = "anyscale/ray:1.9.0rc2-py37-cu102"
    RAY_1_9_0RC2_PY37_CU110 = "anyscale/ray:1.9.0rc2-py37-cu110"
    RAY_1_9_0RC2_PY37_CU111 = "anyscale/ray:1.9.0rc2-py37-cu111"
    RAY_1_9_0RC2_PY37_CU112 = "anyscale/ray:1.9.0rc2-py37-cu112"
    RAY_1_9_0RC2_PY38 = "anyscale/ray:1.9.0rc2-py38"
    RAY_1_9_0RC2_PY38_CU101 = "anyscale/ray:1.9.0rc2-py38-cu101"
    RAY_1_9_0RC2_PY38_CU102 = "anyscale/ray:1.9.0rc2-py38-cu102"
    RAY_1_9_0RC2_PY38_CU110 = "anyscale/ray:1.9.0rc2-py38-cu110"
    RAY_1_9_0RC2_PY38_CU111 = "anyscale/ray:1.9.0rc2-py38-cu111"
    RAY_1_9_0RC2_PY38_CU112 = "anyscale/ray:1.9.0rc2-py38-cu112"
    RAY_1_9_1RC0_PY36 = "anyscale/ray:1.9.1rc0-py36"
    RAY_1_9_1RC0_PY36_CU101 = "anyscale/ray:1.9.1rc0-py36-cu101"
    RAY_1_9_1RC0_PY36_CU102 = "anyscale/ray:1.9.1rc0-py36-cu102"
    RAY_1_9_1RC0_PY36_CU110 = "anyscale/ray:1.9.1rc0-py36-cu110"
    RAY_1_9_1RC0_PY36_CU111 = "anyscale/ray:1.9.1rc0-py36-cu111"
    RAY_1_9_1RC0_PY36_CU112 = "anyscale/ray:1.9.1rc0-py36-cu112"
    RAY_1_9_1RC0_PY37 = "anyscale/ray:1.9.1rc0-py37"
    RAY_1_9_1RC0_PY37_CU101 = "anyscale/ray:1.9.1rc0-py37-cu101"
    RAY_1_9_1RC0_PY37_CU102 = "anyscale/ray:1.9.1rc0-py37-cu102"
    RAY_1_9_1RC0_PY37_CU110 = "anyscale/ray:1.9.1rc0-py37-cu110"
    RAY_1_9_1RC0_PY37_CU111 = "anyscale/ray:1.9.1rc0-py37-cu111"
    RAY_1_9_1RC0_PY37_CU112 = "anyscale/ray:1.9.1rc0-py37-cu112"
    RAY_1_9_1RC0_PY38 = "anyscale/ray:1.9.1rc0-py38"
    RAY_1_9_1RC0_PY38_CU101 = "anyscale/ray:1.9.1rc0-py38-cu101"
    RAY_1_9_1RC0_PY38_CU102 = "anyscale/ray:1.9.1rc0-py38-cu102"
    RAY_1_9_1RC0_PY38_CU110 = "anyscale/ray:1.9.1rc0-py38-cu110"
    RAY_1_9_1RC0_PY38_CU111 = "anyscale/ray:1.9.1rc0-py38-cu111"
    RAY_1_9_1RC0_PY38_CU112 = "anyscale/ray:1.9.1rc0-py38-cu112"
    RAY_NIGHTLY_PY36 = "anyscale/ray:nightly-py36"
    RAY_NIGHTLY_PY36_CU101 = "anyscale/ray:nightly-py36-cu101"
    RAY_NIGHTLY_PY36_CU102 = "anyscale/ray:nightly-py36-cu102"
    RAY_NIGHTLY_PY36_CU110 = "anyscale/ray:nightly-py36-cu110"
    RAY_NIGHTLY_PY36_CU111 = "anyscale/ray:nightly-py36-cu111"
    RAY_NIGHTLY_PY36_CU112 = "anyscale/ray:nightly-py36-cu112"
    RAY_NIGHTLY_PY37 = "anyscale/ray:nightly-py37"
    RAY_NIGHTLY_PY37_CU101 = "anyscale/ray:nightly-py37-cu101"
    RAY_NIGHTLY_PY37_CU102 = "anyscale/ray:nightly-py37-cu102"
    RAY_NIGHTLY_PY37_CU110 = "anyscale/ray:nightly-py37-cu110"
    RAY_NIGHTLY_PY37_CU111 = "anyscale/ray:nightly-py37-cu111"
    RAY_NIGHTLY_PY37_CU112 = "anyscale/ray:nightly-py37-cu112"
    RAY_NIGHTLY_PY38 = "anyscale/ray:nightly-py38"
    RAY_NIGHTLY_PY38_CU101 = "anyscale/ray:nightly-py38-cu101"
    RAY_NIGHTLY_PY38_CU102 = "anyscale/ray:nightly-py38-cu102"
    RAY_NIGHTLY_PY38_CU110 = "anyscale/ray:nightly-py38-cu110"
    RAY_NIGHTLY_PY38_CU111 = "anyscale/ray:nightly-py38-cu111"
    RAY_NIGHTLY_PY38_CU112 = "anyscale/ray:nightly-py38-cu112"

    allowable_values = [RAY_ML_1_2_0, RAY_ML_1_2_0_GPU, RAY_ML_1_3_0, RAY_ML_1_3_0_GPU, RAY_ML_1_3_0_PY36, RAY_ML_1_3_0_PY36_GPU, RAY_ML_1_3_0_PY37, RAY_ML_1_3_0_PY37_GPU, RAY_ML_1_3_0_PY38, RAY_ML_1_3_0_PY38_GPU, RAY_ML_1_4_RC0, RAY_ML_1_4_RC0_GPU, RAY_ML_1_4_RC0_PY36, RAY_ML_1_4_RC0_PY36_GPU, RAY_ML_1_4_RC0_PY37, RAY_ML_1_4_RC0_PY37_GPU, RAY_ML_1_4_RC0_PY38, RAY_ML_1_4_RC0_PY38_GPU, RAY_ML_1_4_RC1, RAY_ML_1_4_RC1_GPU, RAY_ML_1_4_RC1_PY36, RAY_ML_1_4_RC1_PY36_GPU, RAY_ML_1_4_RC1_PY37, RAY_ML_1_4_RC1_PY37_GPU, RAY_ML_1_4_RC1_PY38, RAY_ML_1_4_RC1_PY38_GPU, RAY_ML_1_4_RC2, RAY_ML_1_4_RC2_GPU, RAY_ML_1_4_RC2_PY36, RAY_ML_1_4_RC2_PY36_GPU, RAY_ML_1_4_RC2_PY37, RAY_ML_1_4_RC2_PY37_GPU, RAY_ML_1_4_RC2_PY38, RAY_ML_1_4_RC2_PY38_GPU, RAY_ML_1_4_0, RAY_ML_1_4_0_GPU, RAY_ML_1_4_0_PY36, RAY_ML_1_4_0_PY36_GPU, RAY_ML_1_4_0_PY37, RAY_ML_1_4_0_PY37_GPU, RAY_ML_1_4_0_PY38, RAY_ML_1_4_0_PY38_GPU, RAY_ML_1_4_1, RAY_ML_1_4_1_GPU, RAY_ML_1_4_1_PY36, RAY_ML_1_4_1_PY36_GPU, RAY_ML_1_4_1_PY37, RAY_ML_1_4_1_PY37_GPU, RAY_ML_1_4_1_PY38, RAY_ML_1_4_1_PY38_GPU, RAY_ML_1_5_0, RAY_ML_1_5_0_GPU, RAY_ML_1_5_0_PY36, RAY_ML_1_5_0_PY36_GPU, RAY_ML_1_5_0_PY37, RAY_ML_1_5_0_PY37_GPU, RAY_ML_1_5_0_PY38, RAY_ML_1_5_0_PY38_GPU, RAY_ML_1_5_2, RAY_ML_1_5_2_GPU, RAY_ML_1_5_2_PY36, RAY_ML_1_5_2_PY36_GPU, RAY_ML_1_5_2_PY37, RAY_ML_1_5_2_PY37_GPU, RAY_ML_1_5_2_PY38, RAY_ML_1_5_2_PY38_GPU, RAY_ML_1_6_0, RAY_ML_1_6_0_GPU, RAY_ML_1_6_0_PY36, RAY_ML_1_6_0_PY36_GPU, RAY_ML_1_6_0_PY37, RAY_ML_1_6_0_PY37_GPU, RAY_ML_1_6_0_PY38, RAY_ML_1_6_0_PY38_GPU, RAY_ML_1_7_0, RAY_ML_1_7_0_GPU, RAY_ML_1_7_0_PY36, RAY_ML_1_7_0_PY36_GPU, RAY_ML_1_7_0_PY37, RAY_ML_1_7_0_PY37_GPU, RAY_ML_1_7_0_PY38, RAY_ML_1_7_0_PY38_GPU, RAY_ML_1_7_0RC0, RAY_ML_1_7_0RC0_GPU, RAY_ML_1_7_0RC0_PY36, RAY_ML_1_7_0RC0_PY36_GPU, RAY_ML_1_7_0RC0_PY37, RAY_ML_1_7_0RC0_PY37_GPU, RAY_ML_1_7_0RC0_PY38, RAY_ML_1_7_0RC0_PY38_GPU, RAY_ML_1_8_0, RAY_ML_1_8_0_GPU, RAY_ML_1_8_0_PY36, RAY_ML_1_8_0_PY36_GPU, RAY_ML_1_8_0_PY37, RAY_ML_1_8_0_PY37_GPU, RAY_ML_1_8_0_PY38, RAY_ML_1_8_0_PY38_GPU, RAY_ML_1_9_0_PY36_GPU, RAY_ML_1_9_0_PY37_GPU, RAY_ML_1_9_0_PY38_GPU, RAY_ML_1_9_0RC1_PY36_GPU, RAY_ML_1_9_0RC1_PY37_GPU, RAY_ML_1_9_0RC1_PY38_GPU, RAY_ML_1_9_0RC2_PY36_GPU, RAY_ML_1_9_0RC2_PY37_GPU, RAY_ML_1_9_0RC2_PY38_GPU, RAY_ML_1_9_1RC0_PY36_GPU, RAY_ML_1_9_1RC0_PY37_GPU, RAY_ML_1_9_1RC0_PY38_GPU, RAY_ML_NIGHTLY_PY36_GPU, RAY_ML_NIGHTLY_PY37_GPU, RAY_ML_NIGHTLY_PY38_GPU, RAY_1_2_0, RAY_1_2_0_GPU, RAY_1_3_0, RAY_1_3_0_GPU, RAY_1_3_0_PY36, RAY_1_3_0_PY36_GPU, RAY_1_3_0_PY37, RAY_1_3_0_PY37_GPU, RAY_1_3_0_PY38, RAY_1_3_0_PY38_GPU, RAY_1_4_RC0, RAY_1_4_RC0_GPU, RAY_1_4_RC0_PY36, RAY_1_4_RC0_PY36_GPU, RAY_1_4_RC0_PY37, RAY_1_4_RC0_PY37_GPU, RAY_1_4_RC0_PY38, RAY_1_4_RC0_PY38_GPU, RAY_1_4_RC1, RAY_1_4_RC1_GPU, RAY_1_4_RC1_PY36, RAY_1_4_RC1_PY36_GPU, RAY_1_4_RC1_PY37, RAY_1_4_RC1_PY37_GPU, RAY_1_4_RC1_PY38, RAY_1_4_RC1_PY38_GPU, RAY_1_4_RC2, RAY_1_4_RC2_GPU, RAY_1_4_RC2_PY36, RAY_1_4_RC2_PY36_GPU, RAY_1_4_RC2_PY37, RAY_1_4_RC2_PY37_GPU, RAY_1_4_RC2_PY38, RAY_1_4_RC2_PY38_GPU, RAY_1_4_0, RAY_1_4_0_GPU, RAY_1_4_0_PY36, RAY_1_4_0_PY36_GPU, RAY_1_4_0_PY37, RAY_1_4_0_PY37_GPU, RAY_1_4_0_PY38, RAY_1_4_0_PY38_GPU, RAY_1_4_1, RAY_1_4_1_GPU, RAY_1_4_1_PY36, RAY_1_4_1_PY36_GPU, RAY_1_4_1_PY37, RAY_1_4_1_PY37_GPU, RAY_1_4_1_PY38, RAY_1_4_1_PY38_GPU, RAY_1_5_0, RAY_1_5_0_GPU, RAY_1_5_0_PY36, RAY_1_5_0_PY36_GPU, RAY_1_5_0_PY37, RAY_1_5_0_PY37_GPU, RAY_1_5_0_PY38, RAY_1_5_0_PY38_GPU, RAY_1_5_2, RAY_1_5_2_GPU, RAY_1_5_2_PY36, RAY_1_5_2_PY36_GPU, RAY_1_5_2_PY37, RAY_1_5_2_PY37_GPU, RAY_1_5_2_PY38, RAY_1_5_2_PY38_GPU, RAY_1_6_0, RAY_1_6_0_GPU, RAY_1_6_0_PY36, RAY_1_6_0_PY36_GPU, RAY_1_6_0_PY37, RAY_1_6_0_PY37_GPU, RAY_1_6_0_PY38, RAY_1_6_0_PY38_GPU, RAY_1_7_0, RAY_1_7_0_GPU, RAY_1_7_0_PY36, RAY_1_7_0_PY36_GPU, RAY_1_7_0_PY37, RAY_1_7_0_PY37_GPU, RAY_1_7_0_PY38, RAY_1_7_0_PY38_GPU, RAY_1_7_0RC0, RAY_1_7_0RC0_GPU, RAY_1_7_0RC0_PY36, RAY_1_7_0RC0_PY36_GPU, RAY_1_7_0RC0_PY37, RAY_1_7_0RC0_PY37_GPU, RAY_1_7_0RC0_PY38, RAY_1_7_0RC0_PY38_GPU, RAY_1_8_0, RAY_1_8_0_GPU, RAY_1_8_0_PY36, RAY_1_8_0_PY36_GPU, RAY_1_8_0_PY37, RAY_1_8_0_PY37_GPU, RAY_1_8_0_PY38, RAY_1_8_0_PY38_GPU, RAY_1_9_0_PY36, RAY_1_9_0_PY36_CU101, RAY_1_9_0_PY36_CU102, RAY_1_9_0_PY36_CU110, RAY_1_9_0_PY36_CU111, RAY_1_9_0_PY36_CU112, RAY_1_9_0_PY37, RAY_1_9_0_PY37_CU101, RAY_1_9_0_PY37_CU102, RAY_1_9_0_PY37_CU110, RAY_1_9_0_PY37_CU111, RAY_1_9_0_PY37_CU112, RAY_1_9_0_PY38, RAY_1_9_0_PY38_CU101, RAY_1_9_0_PY38_CU102, RAY_1_9_0_PY38_CU110, RAY_1_9_0_PY38_CU111, RAY_1_9_0_PY38_CU112, RAY_1_9_0RC1_PY36, RAY_1_9_0RC1_PY36_CU101, RAY_1_9_0RC1_PY36_CU102, RAY_1_9_0RC1_PY36_CU110, RAY_1_9_0RC1_PY36_CU111, RAY_1_9_0RC1_PY36_CU112, RAY_1_9_0RC1_PY37, RAY_1_9_0RC1_PY37_CU101, RAY_1_9_0RC1_PY37_CU102, RAY_1_9_0RC1_PY37_CU110, RAY_1_9_0RC1_PY37_CU111, RAY_1_9_0RC1_PY37_CU112, RAY_1_9_0RC1_PY38, RAY_1_9_0RC1_PY38_CU101, RAY_1_9_0RC1_PY38_CU102, RAY_1_9_0RC1_PY38_CU110, RAY_1_9_0RC1_PY38_CU111, RAY_1_9_0RC1_PY38_CU112, RAY_1_9_0RC2_PY36, RAY_1_9_0RC2_PY36_CU101, RAY_1_9_0RC2_PY36_CU102, RAY_1_9_0RC2_PY36_CU110, RAY_1_9_0RC2_PY36_CU111, RAY_1_9_0RC2_PY36_CU112, RAY_1_9_0RC2_PY37, RAY_1_9_0RC2_PY37_CU101, RAY_1_9_0RC2_PY37_CU102, RAY_1_9_0RC2_PY37_CU110, RAY_1_9_0RC2_PY37_CU111, RAY_1_9_0RC2_PY37_CU112, RAY_1_9_0RC2_PY38, RAY_1_9_0RC2_PY38_CU101, RAY_1_9_0RC2_PY38_CU102, RAY_1_9_0RC2_PY38_CU110, RAY_1_9_0RC2_PY38_CU111, RAY_1_9_0RC2_PY38_CU112, RAY_1_9_1RC0_PY36, RAY_1_9_1RC0_PY36_CU101, RAY_1_9_1RC0_PY36_CU102, RAY_1_9_1RC0_PY36_CU110, RAY_1_9_1RC0_PY36_CU111, RAY_1_9_1RC0_PY36_CU112, RAY_1_9_1RC0_PY37, RAY_1_9_1RC0_PY37_CU101, RAY_1_9_1RC0_PY37_CU102, RAY_1_9_1RC0_PY37_CU110, RAY_1_9_1RC0_PY37_CU111, RAY_1_9_1RC0_PY37_CU112, RAY_1_9_1RC0_PY38, RAY_1_9_1RC0_PY38_CU101, RAY_1_9_1RC0_PY38_CU102, RAY_1_9_1RC0_PY38_CU110, RAY_1_9_1RC0_PY38_CU111, RAY_1_9_1RC0_PY38_CU112, RAY_NIGHTLY_PY36, RAY_NIGHTLY_PY36_CU101, RAY_NIGHTLY_PY36_CU102, RAY_NIGHTLY_PY36_CU110, RAY_NIGHTLY_PY36_CU111, RAY_NIGHTLY_PY36_CU112, RAY_NIGHTLY_PY37, RAY_NIGHTLY_PY37_CU101, RAY_NIGHTLY_PY37_CU102, RAY_NIGHTLY_PY37_CU110, RAY_NIGHTLY_PY37_CU111, RAY_NIGHTLY_PY37_CU112, RAY_NIGHTLY_PY38, RAY_NIGHTLY_PY38_CU101, RAY_NIGHTLY_PY38_CU102, RAY_NIGHTLY_PY38_CU110, RAY_NIGHTLY_PY38_CU111, RAY_NIGHTLY_PY38_CU112]  # noqa: E501

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
    }

    attribute_map = {
    }

    def __init__(self, local_vars_configuration=None):  # noqa: E501
        """SUPPORTEDBASEIMAGESENUM - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration
        self.discriminator = None

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SUPPORTEDBASEIMAGESENUM):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SUPPORTEDBASEIMAGESENUM):
            return True

        return self.to_dict() != other.to_dict()
