# coding: utf-8

# flake8: noqa
"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from openapi_client.models.aws_credentials import AWSCredentials
from openapi_client.models.aws_node_options import AWSNodeOptions
from openapi_client.models.aws_tag import AWSTag
from openapi_client.models.aws_tag_specification import AWSTagSpecification
from openapi_client.models.actor_lifetime import ActorLifetime
from openapi_client.models.actor_logs import ActorLogs
from openapi_client.models.actor_status import ActorStatus
from openapi_client.models.actorlogs_response import ActorlogsResponse
from openapi_client.models.add_instance_pool_member import AddInstancePoolMember
from openapi_client.models.anyscale_aws_account import AnyscaleAWSAccount
from openapi_client.models.anyscale_version_response import AnyscaleVersionResponse
from openapi_client.models.anyscaleawsaccount_response import AnyscaleawsaccountResponse
from openapi_client.models.anyscaleversionresponse_response import AnyscaleversionresponseResponse
from openapi_client.models.app_config import AppConfig
from openapi_client.models.app_config_config_schema import AppConfigConfigSchema
from openapi_client.models.appconfig_response import AppconfigResponse
from openapi_client.models.applied_snapshot import AppliedSnapshot
from openapi_client.models.archived_logs_info import ArchivedLogsInfo
from openapi_client.models.archivedlogsinfo_response import ArchivedlogsinfoResponse
from openapi_client.models.autoscaler_credentials import AutoscalerCredentials
from openapi_client.models.autoscalercredentials_response import AutoscalercredentialsResponse
from openapi_client.models.autosync_session_id import AutosyncSessionId
from openapi_client.models.autosyncsessionid_list_response import AutosyncsessionidListResponse
from openapi_client.models.autosyncsessionid_response import AutosyncsessionidResponse
from openapi_client.models.aws_region_and_zones import AwsRegionAndZones
from openapi_client.models.aws_region_info import AwsRegionInfo
from openapi_client.models.awscredentials_response import AwscredentialsResponse
from openapi_client.models.awsregionandzones_response import AwsregionandzonesResponse
from openapi_client.models.baseimagesenum import BASEIMAGESENUM
from openapi_client.models.bank_account_information import BankAccountInformation
from openapi_client.models.base_job import BaseJob
from openapi_client.models.base_job_status import BaseJobStatus
from openapi_client.models.basejob_response import BasejobResponse
from openapi_client.models.batch_response_batched_result_organization_invitation_base import BatchResponseBatchedResultOrganizationInvitationBase
from openapi_client.models.batched_result_organization_invitation_base import BatchedResultOrganizationInvitationBase
from openapi_client.models.billing_information import BillingInformation
from openapi_client.models.block_device_mapping import BlockDeviceMapping
from openapi_client.models.build import Build
from openapi_client.models.build_log_response import BuildLogResponse
from openapi_client.models.build_response import BuildResponse
from openapi_client.models.build_status import BuildStatus
from openapi_client.models.buildlogresponse_response import BuildlogresponseResponse
from openapi_client.models.card import Card
from openapi_client.models.card_id import CardId
from openapi_client.models.card_list_response import CardListResponse
from openapi_client.models.change_password_params import ChangePasswordParams
from openapi_client.models.cloud import Cloud
from openapi_client.models.cloud_config import CloudConfig
from openapi_client.models.cloud_list_response import CloudListResponse
from openapi_client.models.cloud_name_options import CloudNameOptions
from openapi_client.models.cloud_providers import CloudProviders
from openapi_client.models.cloud_response import CloudResponse
from openapi_client.models.cloud_types import CloudTypes
from openapi_client.models.cluster_config import ClusterConfig
from openapi_client.models.cluster_config_with_session_idle_timeout import ClusterConfigWithSessionIdleTimeout
from openapi_client.models.cluster_features import ClusterFeatures
from openapi_client.models.cluster_monitor_response import ClusterMonitorResponse
from openapi_client.models.cluster_status import ClusterStatus
from openapi_client.models.clusterconfig_response import ClusterconfigResponse
from openapi_client.models.clusterconfigwithsessionidletimeout_response import ClusterconfigwithsessionidletimeoutResponse
from openapi_client.models.clusterfeatures_response import ClusterfeaturesResponse
from openapi_client.models.clustermonitorresponse_response import ClustermonitorresponseResponse
from openapi_client.models.clusterstatus_response import ClusterstatusResponse
from openapi_client.models.compute_node_type import ComputeNodeType
from openapi_client.models.compute_template import ComputeTemplate
from openapi_client.models.compute_template_config import ComputeTemplateConfig
from openapi_client.models.compute_template_query import ComputeTemplateQuery
from openapi_client.models.computetemplate_response import ComputetemplateResponse
from openapi_client.models.computetemplateconfig_response import ComputetemplateconfigResponse
from openapi_client.models.create_app_config import CreateAppConfig
from openapi_client.models.create_app_config_configuration_schema import CreateAppConfigConfigurationSchema
from openapi_client.models.create_build import CreateBuild
from openapi_client.models.create_compute_template import CreateComputeTemplate
from openapi_client.models.create_from_github_options import CreateFromGithubOptions
from openapi_client.models.create_nodes_options import CreateNodesOptions
from openapi_client.models.create_organization_invitation import CreateOrganizationInvitation
from openapi_client.models.create_production_job import CreateProductionJob
from openapi_client.models.create_production_service import CreateProductionService
from openapi_client.models.create_response import CreateResponse
from openapi_client.models.create_session_from_snapshot_options import CreateSessionFromSnapshotOptions
from openapi_client.models.create_session_in_db import CreateSessionInDb
from openapi_client.models.create_snapshot_options import CreateSnapshotOptions
from openapi_client.models.create_user import CreateUser
from openapi_client.models.create_user_project_collaborator import CreateUserProjectCollaborator
from openapi_client.models.create_user_project_collaborator_value import CreateUserProjectCollaboratorValue
from openapi_client.models.credit_card_information import CreditCardInformation
from openapi_client.models.decorated_actor import DecoratedActor
from openapi_client.models.decorated_application_template import DecoratedApplicationTemplate
from openapi_client.models.decorated_build import DecoratedBuild
from openapi_client.models.decorated_compute_template import DecoratedComputeTemplate
from openapi_client.models.decorated_compute_template_config import DecoratedComputeTemplateConfig
from openapi_client.models.decorated_job import DecoratedJob
from openapi_client.models.decorated_production_job import DecoratedProductionJob
from openapi_client.models.decorated_production_job_state_transition import DecoratedProductionJobStateTransition
from openapi_client.models.decorated_runtime_env import DecoratedRuntimeEnv
from openapi_client.models.decorated_serve_deployment import DecoratedServeDeployment
from openapi_client.models.decorated_session import DecoratedSession
from openapi_client.models.decoratedactor_list_response import DecoratedactorListResponse
from openapi_client.models.decoratedactor_response import DecoratedactorResponse
from openapi_client.models.decoratedapplicationtemplate_list_response import DecoratedapplicationtemplateListResponse
from openapi_client.models.decoratedapplicationtemplate_response import DecoratedapplicationtemplateResponse
from openapi_client.models.decoratedbuild_list_response import DecoratedbuildListResponse
from openapi_client.models.decoratedbuild_response import DecoratedbuildResponse
from openapi_client.models.decoratedcomputetemplate_list_response import DecoratedcomputetemplateListResponse
from openapi_client.models.decoratedcomputetemplate_response import DecoratedcomputetemplateResponse
from openapi_client.models.decoratedjob_list_response import DecoratedjobListResponse
from openapi_client.models.decoratedjob_response import DecoratedjobResponse
from openapi_client.models.decoratedproductionjob_list_response import DecoratedproductionjobListResponse
from openapi_client.models.decoratedproductionjob_response import DecoratedproductionjobResponse
from openapi_client.models.decoratedproductionjobstatetransition_list_response import DecoratedproductionjobstatetransitionListResponse
from openapi_client.models.decoratedruntimeenv_list_response import DecoratedruntimeenvListResponse
from openapi_client.models.decoratedruntimeenv_response import DecoratedruntimeenvResponse
from openapi_client.models.decoratedservedeployment_list_response import DecoratedservedeploymentListResponse
from openapi_client.models.decoratedservedeployment_response import DecoratedservedeploymentResponse
from openapi_client.models.decoratedsession_list_response import DecoratedsessionListResponse
from openapi_client.models.decoratedsession_response import DecoratedsessionResponse
from openapi_client.models.deployment_status import DeploymentStatus
from openapi_client.models.dismissal_type import DismissalType
from openapi_client.models.ebs_block_device import EbsBlockDevice
from openapi_client.models.error import Error
from openapi_client.models.execute_command_response import ExecuteCommandResponse
from openapi_client.models.execute_interactive_command_options import ExecuteInteractiveCommandOptions
from openapi_client.models.execute_shell_command_options import ExecuteShellCommandOptions
from openapi_client.models.executecommandresponse_response import ExecutecommandresponseResponse
from openapi_client.models.external_service_status import ExternalServiceStatus
from openapi_client.models.external_service_status_response import ExternalServiceStatusResponse
from openapi_client.models.external_terminal_command import ExternalTerminalCommand
from openapi_client.models.externalservicestatusresponse_response import ExternalservicestatusresponseResponse
from openapi_client.models.feature_compatibility import FeatureCompatibility
from openapi_client.models.feature_flag_response import FeatureFlagResponse
from openapi_client.models.featureflagresponse_response import FeatureflagresponseResponse
from openapi_client.models.gcp_node_options import GCPNodeOptions
from openapi_client.models.github_project import GithubProject
from openapi_client.models.githubproject_response import GithubprojectResponse
from openapi_client.models.http_validation_error import HTTPValidationError
from openapi_client.models.ha_job_goal_states import HaJobGoalStates
from openapi_client.models.ha_job_states import HaJobStates
from openapi_client.models.ha_job_type import HaJobType
from openapi_client.models.head_ip import HeadIp
from openapi_client.models.headip_response import HeadipResponse
from openapi_client.models.historical_cost_granularity import HistoricalCostGranularity
from openapi_client.models.historical_costs import HistoricalCosts
from openapi_client.models.historicalcosts_list_response import HistoricalcostsListResponse
from openapi_client.models.iam_instance_profile_specification import IamInstanceProfileSpecification
from openapi_client.models.instance import Instance
from openapi_client.models.instance_external_ip import InstanceExternalIP
from openapi_client.models.instance_id import InstanceID
from openapi_client.models.instance_internal_ip import InstanceInternalIP
from openapi_client.models.instance_is_running import InstanceIsRunning
from openapi_client.models.instance_is_terminated import InstanceIsTerminated
from openapi_client.models.instance_list_response import InstanceListResponse
from openapi_client.models.instance_pool_member import InstancePoolMember
from openapi_client.models.instance_response import InstanceResponse
from openapi_client.models.instance_status import InstanceStatus
from openapi_client.models.instanceexternalip_response import InstanceexternalipResponse
from openapi_client.models.instanceid_response import InstanceidResponse
from openapi_client.models.instanceinternalip_response import InstanceinternalipResponse
from openapi_client.models.instanceisrunning_response import InstanceisrunningResponse
from openapi_client.models.instanceisterminated_response import InstanceisterminatedResponse
from openapi_client.models.instancepoolmember_response import InstancepoolmemberResponse
from openapi_client.models.internal_production_job import InternalProductionJob
from openapi_client.models.internalproductionjob_response import InternalproductionjobResponse
from openapi_client.models.invoice import Invoice
from openapi_client.models.invoice_list_response import InvoiceListResponse
from openapi_client.models.invoice_status import InvoiceStatus
from openapi_client.models.invoices_query import InvoicesQuery
from openapi_client.models.job_access import JobAccess
from openapi_client.models.job_run_type import JobRunType
from openapi_client.models.job_state_log_level_types import JobStateLogLevelTypes
from openapi_client.models.jobs_logs import JobsLogs
from openapi_client.models.jobslogs_response import JobslogsResponse
from openapi_client.models.json_patch_operation import JsonPatchOperation
from openapi_client.models.list_response_metadata import ListResponseMetadata
from openapi_client.models.log_level_types import LogLevelTypes
from openapi_client.models.login_user_params import LoginUserParams
from openapi_client.models.logs_output import LogsOutput
from openapi_client.models.logsoutput_response import LogsoutputResponse
from openapi_client.models.metric_period import MetricPeriod
from openapi_client.models.mini_build import MiniBuild
from openapi_client.models.mini_cloud import MiniCloud
from openapi_client.models.mini_cluster import MiniCluster
from openapi_client.models.mini_compute_template import MiniComputeTemplate
from openapi_client.models.mini_job_run import MiniJobRun
from openapi_client.models.mini_namespace import MiniNamespace
from openapi_client.models.mini_project import MiniProject
from openapi_client.models.mini_runtime_environment import MiniRuntimeEnvironment
from openapi_client.models.mini_user import MiniUser
from openapi_client.models.network_interface import NetworkInterface
from openapi_client.models.nodes_options import NodesOptions
from openapi_client.models.non_terminated_nodes_options import NonTerminatedNodesOptions
from openapi_client.models.onboarding_user_cards_query import OnboardingUserCardsQuery
from openapi_client.models.organization import Organization
from openapi_client.models.organization_availability import OrganizationAvailability
from openapi_client.models.organization_collaborator import OrganizationCollaborator
from openapi_client.models.organization_invitation import OrganizationInvitation
from openapi_client.models.organization_invitation_base import OrganizationInvitationBase
from openapi_client.models.organization_permission_level import OrganizationPermissionLevel
from openapi_client.models.organization_project_collaborator import OrganizationProjectCollaborator
from openapi_client.models.organization_project_collaborator_value import OrganizationProjectCollaboratorValue
from openapi_client.models.organization_response import OrganizationResponse
from openapi_client.models.organizationavailability_response import OrganizationavailabilityResponse
from openapi_client.models.organizationcollaborator_list_response import OrganizationcollaboratorListResponse
from openapi_client.models.organizationinvitation_list_response import OrganizationinvitationListResponse
from openapi_client.models.organizationinvitation_response import OrganizationinvitationResponse
from openapi_client.models.organizationinvitationbase_response import OrganizationinvitationbaseResponse
from openapi_client.models.organizationprojectcollaborator_list_response import OrganizationprojectcollaboratorListResponse
from openapi_client.models.permission_level import PermissionLevel
from openapi_client.models.product_type import ProductType
from openapi_client.models.production_job_config import ProductionJobConfig
from openapi_client.models.production_job_state_transition import ProductionJobStateTransition
from openapi_client.models.project import Project
from openapi_client.models.project_base import ProjectBase
from openapi_client.models.project_collaborator import ProjectCollaborator
from openapi_client.models.project_collaborator_value import ProjectCollaboratorValue
from openapi_client.models.project_collaborators_put_message import ProjectCollaboratorsPutMessage
from openapi_client.models.project_create_message import ProjectCreateMessage
from openapi_client.models.project_default_session_name import ProjectDefaultSessionName
from openapi_client.models.project_delete_message import ProjectDeleteMessage
from openapi_client.models.project_list_response import ProjectListResponse
from openapi_client.models.project_patch_message import ProjectPatchMessage
from openapi_client.models.project_response import ProjectResponse
from openapi_client.models.projectbase_response import ProjectbaseResponse
from openapi_client.models.projectcollaborator_list_response import ProjectcollaboratorListResponse
from openapi_client.models.projectdefaultsessionname_response import ProjectdefaultsessionnameResponse
from openapi_client.models.provider_metadata import ProviderMetadata
from openapi_client.models.providermetadata_response import ProvidermetadataResponse
from openapi_client.models.python_modules import PythonModules
from openapi_client.models.query_pool_size import QueryPoolSize
from openapi_client.models.ray_runtime_env_config import RayRuntimeEnvConfig
from openapi_client.models.request_instance_pool_member import RequestInstancePoolMember
from openapi_client.models.request_organization_identifiers_params import RequestOrganizationIdentifiersParams
from openapi_client.models.request_password_reset_params import RequestPasswordResetParams
from openapi_client.models.reset_password_params import ResetPasswordParams
from openapi_client.models.resource_historical_costs import ResourceHistoricalCosts
from openapi_client.models.resources import Resources
from openapi_client.models.s3_download_location import S3DownloadLocation
from openapi_client.models.sso_login_info import SSOLoginInfo
from openapi_client.models.supportedbaseimagesenum import SUPPORTEDBASEIMAGESENUM
from openapi_client.models.serve_deployment_grafana_dashboard_status import ServeDeploymentGrafanaDashboardStatus
from openapi_client.models.serve_deployment_logs import ServeDeploymentLogs
from openapi_client.models.serve_replica import ServeReplica
from openapi_client.models.serve_replica_metric import ServeReplicaMetric
from openapi_client.models.servedeploymentlogs_response import ServedeploymentlogsResponse
from openapi_client.models.server_session_token import ServerSessionToken
from openapi_client.models.servereplica_list_response import ServereplicaListResponse
from openapi_client.models.serversessiontoken_response import ServersessiontokenResponse
from openapi_client.models.service_account import ServiceAccount
from openapi_client.models.service_usage import ServiceUsage
from openapi_client.models.session import Session
from openapi_client.models.session_access import SessionAccess
from openapi_client.models.session_autosync_sessions_update_message import SessionAutosyncSessionsUpdateMessage
from openapi_client.models.session_command import SessionCommand
from openapi_client.models.session_command_finish_options import SessionCommandFinishOptions
from openapi_client.models.session_command_id import SessionCommandId
from openapi_client.models.session_command_types import SessionCommandTypes
from openapi_client.models.session_create_message import SessionCreateMessage
from openapi_client.models.session_delete_message import SessionDeleteMessage
from openapi_client.models.session_describe import SessionDescribe
from openapi_client.models.session_details import SessionDetails
from openapi_client.models.session_event import SessionEvent
from openapi_client.models.session_event_cause import SessionEventCause
from openapi_client.models.session_event_types import SessionEventTypes
from openapi_client.models.session_execute_message import SessionExecuteMessage
from openapi_client.models.session_finish_command_message import SessionFinishCommandMessage
from openapi_client.models.session_history_item import SessionHistoryItem
from openapi_client.models.session_id import SessionId
from openapi_client.models.session_kill_command_message import SessionKillCommandMessage
from openapi_client.models.session_list_response import SessionListResponse
from openapi_client.models.session_patch_message import SessionPatchMessage
from openapi_client.models.session_response import SessionResponse
from openapi_client.models.session_ssh_key import SessionSshKey
from openapi_client.models.session_starting_up_data import SessionStartingUpData
from openapi_client.models.session_state import SessionState
from openapi_client.models.session_state_change_message import SessionStateChangeMessage
from openapi_client.models.session_state_data import SessionStateData
from openapi_client.models.session_stopping_data import SessionStoppingData
from openapi_client.models.session_up_options import SessionUpOptions
from openapi_client.models.sessioncommand_list_response import SessioncommandListResponse
from openapi_client.models.sessioncommandid_response import SessioncommandidResponse
from openapi_client.models.sessiondescribe_response import SessiondescribeResponse
from openapi_client.models.sessiondetails_response import SessiondetailsResponse
from openapi_client.models.sessionevent_list_response import SessioneventListResponse
from openapi_client.models.sessionhistoryitem_list_response import SessionhistoryitemListResponse
from openapi_client.models.sessionid_response import SessionidResponse
from openapi_client.models.sessionsshkey_response import SessionsshkeyResponse
from openapi_client.models.set_node_tags_options import SetNodeTagsOptions
from openapi_client.models.setup_initialize_session_options import SetupInitializeSessionOptions
from openapi_client.models.snapshot import Snapshot
from openapi_client.models.snapshot_create import SnapshotCreate
from openapi_client.models.snapshot_create_message import SnapshotCreateMessage
from openapi_client.models.snapshot_delete import SnapshotDelete
from openapi_client.models.snapshot_delete_message import SnapshotDeleteMessage
from openapi_client.models.snapshot_files import SnapshotFiles
from openapi_client.models.snapshot_id import SnapshotID
from openapi_client.models.snapshot_list import SnapshotList
from openapi_client.models.snapshot_list_item import SnapshotListItem
from openapi_client.models.snapshot_patch_message import SnapshotPatchMessage
from openapi_client.models.snapshot_project_file import SnapshotProjectFile
from openapi_client.models.snapshot_response import SnapshotResponse
from openapi_client.models.snapshotcreate_response import SnapshotcreateResponse
from openapi_client.models.snapshotdelete_response import SnapshotdeleteResponse
from openapi_client.models.snapshotfiles_response import SnapshotfilesResponse
from openapi_client.models.snapshotid_response import SnapshotidResponse
from openapi_client.models.snapshotlist_response import SnapshotlistResponse
from openapi_client.models.socket_message_schemas import SocketMessageSchemas
from openapi_client.models.socket_message_types import SocketMessageTypes
from openapi_client.models.socketmessageschemas_response import SocketmessageschemasResponse
from openapi_client.models.socketmessagetypes_response import SocketmessagetypesResponse
from openapi_client.models.ssologininfo_response import SsologininfoResponse
from openapi_client.models.start_empty_session_response import StartEmptySessionResponse
from openapi_client.models.start_session_options import StartSessionOptions
from openapi_client.models.startemptysessionresponse_response import StartemptysessionresponseResponse
from openapi_client.models.stop_session_options import StopSessionOptions
from openapi_client.models.take_snapshot_options import TakeSnapshotOptions
from openapi_client.models.text_query import TextQuery
from openapi_client.models.update_compute_template import UpdateComputeTemplate
from openapi_client.models.update_organization_collaborator import UpdateOrganizationCollaborator
from openapi_client.models.update_project_collaborator import UpdateProjectCollaborator
from openapi_client.models.upload_session_command_logs_locations import UploadSessionCommandLogsLocations
from openapi_client.models.uploadsessioncommandlogslocations_response import UploadsessioncommandlogslocationsResponse
from openapi_client.models.user_info import UserInfo
from openapi_client.models.user_resend_email_options import UserResendEmailOptions
from openapi_client.models.userinfo_response import UserinfoResponse
from openapi_client.models.validation_error import ValidationError
from openapi_client.models.visibility import Visibility
from openapi_client.models.wait_until_stopped_options import WaitUntilStoppedOptions
from openapi_client.models.web_terminal import WebTerminal
from openapi_client.models.webterminal_list_response import WebterminalListResponse
from openapi_client.models.webterminal_response import WebterminalResponse
from openapi_client.models.worker_node_type import WorkerNodeType
from openapi_client.models.write_cloud import WriteCloud
from openapi_client.models.write_cluster_config import WriteClusterConfig
from openapi_client.models.write_project import WriteProject
from openapi_client.models.write_session import WriteSession
from openapi_client.models.write_snapshot import WriteSnapshot
